/*******************************************************************************
 * Copyright (c) {2007, 2008} Engineering Ingegneria Informatica S.p.A.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors: Andrea Zoppello (Engineering) - initial API and implementation
 *******************************************************************************/
package org.eclipse.stp.im.runtime.property.listeners;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.stp.im.runtime.IProperty;

public class ExtractParamsMapHandlerListener implements IPropertyListener {
	private String matchRegexp = "\\$\\w+";
	public static String PLACE_HOLDER_SIGN = "$".intern();
	
	public void handleUpdate(String sourcePropertyValue, String dependentPropertyName, IProperty destinationServiceBindingProperty, EAnnotation ea ) {
		
		// 
		// In this implementation the property that has changes is a string 
		// where parameters are identified by string line $paramName
		//
		
		List<String> parametersList = getStringParameterNames(sourcePropertyValue);
		

		if (parametersList.size() >= 0){
			//System.out.println(" Cannot handle a number of value less then 0");
			String basePropertyName = null;
			for (String aParamName : parametersList) {
				basePropertyName = dependentPropertyName + "["+aParamName+"]";
				String propertyToFoundInEAnnotation = basePropertyName+"."+destinationServiceBindingProperty.getMapFieldList().get(0);
				
				// -------------------------------------------------------
				// If the eAnnotation contains already just the value
				// keep it
				// --------------------------------------------------------
				
				if (ea.getDetails().get(propertyToFoundInEAnnotation) == null){
					String destPropertyNameInAnnotation = null;	
					//--------------------------------------------------------
					// Write a "Map Entry" in the eAnnotation
					// -------------------------------------------------------
					
					for (String propOfMap : destinationServiceBindingProperty.getMapFieldList()) {
					
						destPropertyNameInAnnotation = basePropertyName + "." + propOfMap;
						ea.getDetails().put(destPropertyNameInAnnotation, "");
					
					}
				}else{
					// -----------
					// -----------
				}
			}
			
			
			// ------------------------------------------------------------------------------
			//
			// Se ho rimosso delle property Devo Gestirlo
			// 
			// ------------------------------------------------------------------------------
			
			String parName = null;

			List<String> keyToRemove = new ArrayList<String>();
			
			for (String s : ea.getDetails().keySet() ){
				
				if (s.startsWith(dependentPropertyName)){
					parName = s.substring(s.indexOf("[") +1, s.indexOf("]"));
					
					
					if (!parametersList.contains(parName))
						keyToRemove.add(s);
			
				} // -- end if (s.startsWith(dependentPropertyName))
			
			} // -- end for (String s : ea.getDetails().keySet() )
			
			for ( String s : keyToRemove){
			
				ea.getDetails().removeKey(s);
			
			}
			
		}// end if (changedValueInt > 0)
	}
	
	public List<String> getStringParameterNames(String propertyValue) {
        Pattern pattern = Pattern.compile(matchRegexp);
        Matcher matcher = 
        	pattern.matcher(propertyValue);
        String placeHolder;
        List<String> toReturn = new ArrayList<String>();
		for (int idx = 1; matcher.find(); idx++) {
			placeHolder = matcher.group();
			placeHolder = placeHolder.substring(
				placeHolder.indexOf(propertyValue)+1);
			toReturn.add(placeHolder);
		}
		return toReturn;
	}
}
